// T4CEDemoDlg.cpp : implementation file
//

#include "stdafx.h"
#include "T4CEDemo.h"
#include "T4CEDemoDlg.h"
#include "T4W.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


const int TIMER_ID = 1;
const int TIMER_INIT = 500;
const int TIMER_SEARCH = 50;

bool IsPortrait()
{
	RECT Rect;
	::GetWindowRect(::GetDesktopWindow(),&Rect);
	int Height = Rect.bottom-Rect.top;
	int Width = Rect.right-Rect.left;
	return Height > Width;
}

struct CTagInfo
{
	LPCSTR Name;
	LPCTSTR _tName;
	TCHAR TypeChar;
	int TypeID;
	int TypeMask;
	int IDLength;
	int IDOffset;
   bool RW;
};

const CTagInfo TagInfo[] =
{
	{ "IDRO-A", _T("IDRO-A"), 'A', T4_TYPE_IDRO_A, T4_MSK_IDRO_A, 5, 0, false },
	{ "IDRW-B", _T("IDRW-B"), 'B', T4_TYPE_IDRW_B, T4_MSK_IDRW_B, 4, 1, true  },
	{ "IDRW-C", _T("IDRW-C"), 'C', T4_TYPE_IDRW_C, T4_MSK_IDRW_C, 4, 1, true  },
	{ "IDRW-D", _T("IDRW-D"), 'D', T4_TYPE_IDRW_D, T4_MSK_IDRW_D, 4, 1, true  },
	{ "IDRW-E", _T("IDRW-E"), 'E', T4_TYPE_IDRW_E, T4_MSK_IDRW_E, 4, 1, true  },
	{ "IDRO-G", _T("IDRO-G"), 'G', T4_TYPE_IDRO_G, T4_MSK_IDRO_G, 8, 0, false },
	{ "IDRO-H", _T("IDRO-H"), 'H', T4_TYPE_IDRO_H, T4_MSK_IDRO_H, 8, 0, false },
	{ NULL    , NULL        ,              0,             0, 0, 0, false }
};

/////////////////////////////////////////////////////////////////////////////
// CT4CEDemoDlg dialog

CT4CEDemoDlg::CT4CEDemoDlg(CWnd* pParent /*=NULL*/)
: CDialog(IsPortrait() ? CT4CEDemoDlg::IDD_V : CT4CEDemoDlg::IDD_H, pParent)
{
	//{{AFX_DATA_INIT(CT4CEDemoDlg)
	m_cbRead = FALSE;
	m_cbWrite = FALSE;
	m_cbSave = FALSE;
	m_rbWriteType = 0;
	m_cbTagType = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CT4CEDemoDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CT4CEDemoDlg)
	DDX_Control(pDX, IDC_ST_DATA, m_stReadData);
	DDX_Control(pDX, IDC_ST_TYPE, m_stType);
	DDX_Control(pDX, IDC_ST_STATUS, m_stStatus);
	DDX_Control(pDX, IDC_ST_ID, m_stID);
	DDX_Control(pDX, IDC_EB_WRDATA, m_ebWriteData);
	DDX_Check(pDX, IDC_CK_READ, m_cbRead);
	DDX_Check(pDX, IDC_CK_WRITE, m_cbWrite);
	DDX_Check(pDX, IDC_CK_SAVE, m_cbSave);
	DDX_Radio(pDX, IDC_RB_WREB, m_rbWriteType);
	DDX_CBIndex(pDX, IDC_CB_TAGTYPE, m_cbTagType);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CT4CEDemoDlg, CDialog)
	//{{AFX_MSG_MAP(CT4CEDemoDlg)
	ON_CBN_SELCHANGE(IDC_CB_TAGTYPE, OnSelChangeCbTagtype)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CT4CEDemoDlg message handlers

BOOL CT4CEDemoDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// we activated the center option of the dialog
	// CenterWindow(GetDesktopWindow());	// center to the hpc screen

	// do a precheck for the reader
	if (T4FindCOMPort() == NULL)
		m_stStatus.SetWindowText(_T("No reader found!"));

	IsInit = false;
	SetTimer(TIMER_ID,TIMER_INIT,NULL);

	// Don't set focus on first control
	return FALSE;
}

void CT4CEDemoDlg::OnSelChangeCbTagtype() 
{
    UpdateData(TRUE);
	if (m_cbTagType < 0 || m_cbTagType > 7)
		return;	// Souldn't occur

	int Status;
	// Which transponder types does the reader support?
	BYTE Version[5];
	Status = T4VersionTagReader(Version);
	if (Status != T4_NOERROR)
	{
		ShowFString(_T("%s"),T4GetStatusString(Status));
		return;
	}
	// The last byte of the version info reflects the supported transponders
	BYTE SupportedTags = Version[4];
	if (m_cbTagType == 0)
	{
		// Selecting "Auto" means to read any transponder type but not IDRW-E
		SupportedTags &= ~T4_MSK_IDRW_E;
		Status = T4SetTagDriver(SupportedTags);
		if (Status != T4_NOERROR)
		{
			ShowFString(_T("%s"),T4GetStatusString(Status));
			return;
		}
		// Collect all supported transponders and create a string
		TCHAR TagsBuffer[30];
		TCHAR *p = TagsBuffer;
		const CTagInfo *pTagInfo;
		for (pTagInfo = TagInfo; pTagInfo->Name != NULL; pTagInfo++)
			if (SupportedTags & pTagInfo->TypeMask)
			{
				if (p != TagsBuffer)
					*p++ = '/';
				*p++ = pTagInfo->TypeChar;
			}
		*p = 0;
		ShowFString(_T("Searching for %s"),TagsBuffer);
	}
	else
	{
		// we are only searching for one specific transponder type
		// prepare a pointer into TagInfo
		const CTagInfo *pTagInfo = &TagInfo[m_cbTagType-1];
		if ((SupportedTags & pTagInfo->TypeMask) == 0)
		{
			ShowFString(_T("Not supported"));
			return;
		}
		int Status = T4SetTagDriver(pTagInfo->TypeMask);
		if (Status != T4_NOERROR)
		{
			ShowFString(_T("%s"),T4GetStatusString(Status));
			return;
		}
		ShowFString(_T("Searching for %s"),pTagInfo->_tName);
	}
}

void CT4CEDemoDlg::OnTimer(UINT nIDEvent) 
{
	CDialog::OnTimer(nIDEvent);

	if (nIDEvent != TIMER_ID)
		return;

	if (!IsInit)
	{
		// if not initialized, try to init

		// prevent us from getting many timer events during init
		KillTimer(TIMER_ID);

		// get port name/is the CF card inserted?
		LPCTSTR Port = T4FindCOMPort();

		if (Port == NULL)
		{
			// no reader found
			// still wait till reader is inserted
			SetTimer(TIMER_ID,TIMER_INIT,NULL);
			return;
		}

		ShowFString(_T("Initializing %s..."),Port);
		m_stStatus.UpdateWindow();

		int Error;

		Error = T4Init(Port);
		if (Error == T4_NOERROR)
		{
			Error = T4ConfigurationReset();
			if (Error == T4_NOERROR)
			{
				// prepare for sensefull reading a IDRW-E, ignore error
				T4SetTagOptions(T4_TYPE_IDRW_E,0x38);
				OnSelChangeCbTagtype();
			}
		}

		if (Error != T4_NOERROR)
		{
			T4DeInit();
			m_stStatus.SetWindowText(T4GetStatusString(Error));
			SetTimer(TIMER_ID,TIMER_INIT,NULL);
			return;
		}

		// yes, we are initialized!
		IsInit = true;
		// init timeout for status texts
		StTimeout = 1;
		// init timer
		SetTimer(TIMER_ID,TIMER_SEARCH,NULL);

		// start cyclic polling of the reader
		// use the standard password from IDRW-C
		BYTE PW[4] = { 'M','I','K','R' };
		Error = T4InitTagPoll(PW);
		return;
	}

	// do some sensefull operations with the reader

	const int TransferCnt = 4;
	const int IDMaxByteCnt = 8;
	const int ReadWordCnt = 4;
	const int WriteWordCnt = 4;
	const int ReadByteCnt = ReadWordCnt*4;
	const int WriteByteCnt = WriteWordCnt*4;

	char StrID     [IDMaxByteCnt*3-1 +1];	// save max 5 hex numbers separated by spaces
	char StrTagType[6                +1];	// save max 6 characters
	char StrDataHex[ReadByteCnt*3-1  +1];	// save max 16 hex numbers separated by spaces
	char StrDataAsc[ReadByteCnt      +1];	// save max 16 characters

	int Error;
	BYTE TagType;
	BYTE IDData[IDMaxByteCnt];
    BYTE RdData[ReadByteCnt];
    BYTE WrData[WriteByteCnt];

	FILE *SaveFile;

	Error = T4DoTagPoll(&TagType,IDData);

	switch (Error)
	{
	case T4_NOTAG:
		if (StTimeout > 0)
		{
			if (StTimeout == 1)
			{
				m_stStatus.SetWindowText(_T("Searching..."));
				m_stID.SetWindowText(_T(""));
				m_stReadData.SetWindowText(_T(""));
				m_stType.SetWindowText(_T(""));
			}
			StTimeout--;
		}
		break;

	case T4_NOERROR:
		m_stStatus.SetWindowText(_T("Found Tag"));
		StTimeout = 100;
		break;

	default:
		goto Error;
	}

    UpdateData(TRUE);

    // did we found a transponder?
	if (Error != T4_NOERROR && Error != T4_WRONGTAGPW)
		// not really
		goto Error;


	// what type of transponder are we talking about?
    const CTagInfo *pTagInfo;
	for (pTagInfo = TagInfo; pTagInfo->Name != NULL; pTagInfo++)
		if (TagType == pTagInfo->TypeID)
			break;
	if (pTagInfo->Name == NULL)
	{
		m_stType.SetWindowText(_T("Unknown"));
		return;
	}
	// create the appropriate strings for displaying and saving
	m_stType.SetWindowText(pTagInfo->_tName);
	strcpy(StrTagType,pTagInfo->Name);
	m_stID.SetWindowText(_tGetHexString(&IDData[pTagInfo->IDOffset],pTagInfo->IDLength));
	strcpy(StrID,GetHexString(&IDData[pTagInfo->IDOffset],pTagInfo->IDLength));
    if (pTagInfo->TypeID == T4_TYPE_IDRO_G)
    {
		// show some extra information for a IDRO-G

		BYTE BinBuf[8]; // 40 bits
		TCHAR ManuCode[20+1];
		TCHAR IDCode[20+1];

		memset(BinBuf,0x00,sizeof(BinBuf));
		T4CopyBits(IDData,BinBuf,26,0,38,true);
		_tcscpy(IDCode,T4GetDecString(BinBuf,38));

		memset(BinBuf,0x00,sizeof(BinBuf));
		T4CopyBits(IDData,BinBuf,16,0,10,true);
		_tcscpy(ManuCode,T4GetDecString(BinBuf,10));

		TCHAR Buffer[200];
		_stprintf(Buffer,_T("ID : %s; Manuf.: %s; Animal: %s"),
                        IDCode,
                        ManuCode,
                        IDData[0] & 0x01 ? _T("Yes") : _T("No"));
		m_stReadData.SetWindowText(Buffer);
	}

	strcpy(StrDataHex,"");
	strcpy(StrDataAsc,"");

	if ((m_cbRead || m_cbWrite) && !pTagInfo->RW)
	{
		m_stReadData.SetWindowText(_T("Tag has no data space!"));
		goto OK;
	}

	if (m_cbRead)
	{
		for (BYTE Adr = 0; Adr < ReadWordCnt; Adr += TransferCnt)
		{
			BYTE Cnt = ReadWordCnt - Adr;
			if (Cnt > TransferCnt)
				Cnt = TransferCnt;
			Error = T4ReadData(Adr,Cnt,&RdData[Adr*4]);
			if (Error != T4_NOERROR)
				goto Error;
		}

		int i;
		// for viewing, create a wide char string out of the data
		TCHAR CharVal[ReadByteCnt+1];
		for (i=0; i<ReadByteCnt; i++)
			CharVal[i] = RdData[i] < ' ' ? ' ' : RdData[i];
		CharVal[i] = 0;
		TCHAR Buffer[200];
		_stprintf(Buffer,_T("%s (\"%s\")"),_tGetHexString(RdData,sizeof(RdData)),CharVal);
		m_stReadData.SetWindowText(Buffer);

		// prepare strings for saving in file
		strcpy(StrDataHex,GetHexString(RdData,sizeof(RdData)));
		for (i=0; i<ReadByteCnt; i++)
			StrDataAsc[i] = RdData[i] < ' ' ? ' ' : (char)RdData[i];
		StrDataAsc[i] = 0;
	}

	if (m_cbWrite)
	{
		BYTE i;

		switch (m_rbWriteType)
		{
		case 0:	// Write content of Editbox
			TCHAR Buffer[WriteByteCnt+1];
			memset(Buffer,0x00,sizeof(Buffer));
			m_ebWriteData.GetWindowText(Buffer,WriteByteCnt+1);
			for (i=0; i<WriteByteCnt; i++)
				WrData[i] = (BYTE)Buffer[i];
			// We can store 16 chars: 0 to 15 with nul termination, 16 without
			break;
		case 1:	// Write 00
			memset(WrData,0x00,WriteByteCnt);
			break;
		case 2:	// Write FF
			memset(WrData,0xFF,WriteByteCnt);
			break;
		case 3:	// Write Enum
			for (i=0; i<WriteByteCnt; i++)
				WrData[i] = i;
			break;
		}

		for (BYTE Adr = 0; Adr < WriteWordCnt; Adr += TransferCnt)
		{
			BYTE Cnt = WriteWordCnt - Adr;
			if (Cnt > TransferCnt)
				Cnt = TransferCnt;
			Error = T4WriteData(Adr,Cnt,&WrData[Adr*4]);
			if (Error != T4_NOERROR)
				goto Error;
		}
	}
OK:
	MessageBeep(MB_OK);

	if (!m_cbSave)
		return;

	// if requested save data to file

	SaveFile = fopen("TAGDATA.TXT","a");
	if (SaveFile == NULL)
		return;
		{
		// Get time
		CTime Time = CTime::GetCurrentTime();
		// Save it
		fprintf(SaveFile,"%02d.%02d.%02d\t%02d:%02d:%02d\t",
							Time.GetDay(),
							Time.GetMonth(),
							Time.GetYear() % 100,
							Time.GetHour(),
							Time.GetMinute(),
							Time.GetSecond());
		}
	fprintf(SaveFile,"%s\t%s\t%s\t%s\n",StrTagType,StrID,StrDataHex,StrDataAsc);
	fclose(SaveFile);
	return;

Error:
	if (Error == T4_NOTINIT)
	{
		MessageBeep(MB_OK);
		KillTimer(TIMER_ID);
		T4DeInit();
		IsInit = FALSE;
		SetTimer(TIMER_ID,TIMER_INIT,NULL);
		m_stStatus.SetWindowText(_T("Reader removed!"));
		return;
	}
	if (Error != T4_NOTAG)
		ShowStatus(Error);
}

/****** Helper Functions ****************************************************/

TCHAR *CT4CEDemoDlg::_tGetHexString(BYTE *Data,int Length)
{
	static TCHAR HexString[500+1];
	TCHAR *Dest;

	if (Length > 160)
		Length = 160;
	Dest = HexString;
	for (int i=0; i<Length; i++)
	{
		_stprintf(Dest,_T("%02X"),(int)Data[i]);
		Dest += 2;
		if (i<Length-1)
		{
			_tcscat(Dest,_T(" "));
			Dest++;
		}
	}
   return HexString;
}

char *CT4CEDemoDlg::GetHexString(BYTE *Data,int Length)
{
	static char HexString[500+1];
	char *Dest;

	if (Length > 160)
		Length = 160;
	Dest = HexString;
	for (int i=0; i<Length; i++)
	{
		sprintf(Dest,"%02X",(int)Data[i]);
		Dest += 2;
		if (i<Length-1)
		{
			strcat(Dest," ");
			Dest++;
		}
	}
   return HexString;
}

void CT4CEDemoDlg::ShowFString(LPCTSTR Format,...)
{
	TCHAR String[100];
	va_list	ArgList;
	va_start(ArgList,Format);
	_vstprintf(String,Format,ArgList);
	m_stStatus.SetWindowText(String);
	va_end(ArgList);
	StTimeout = 100;	// 50ms * 100 = 5 seconds display timeout
}

void CT4CEDemoDlg::ShowStatus(int Status)
{
	ShowFString(_T("%s"),T4GetStatusString(Status));
}

